function [B] = MITOR(X_train,Y_train,adj,lr,alpha,beta)
% ADMM optimizer for model MITOR-II in the paper "Incomplete Label 
% Multi-Task Ordinal Regression for Spatial Event Scale Forecastingporal 
% Event Forecasting".
% The input of the function is as follows:
% X_train: training input data, a 3-D martix where each dimension represents [task, sample, feature vector]
% Y_train: the labels, a 2-D martix where each dimension represents [task, sample]
% adj: the adjacency matrix of the tasks, where adj(i,j)=1 stands for task i and task j are neighbors.
% lr: the learning rate for gradient descent on W and Theta(T)
% The output of the function is as follows:
% B: The result B is the parameters learned by the model

values = unique(Y_train);

k=size(values,1);

TRAIN_size=size(Y_train,2);
TASK_num= size(Y_train, 1);

%%


BIG = inf;
SMALL = -inf;

% d=demension of feature vector
d=size(X_train,3);

R=zeros(TASK_num,TASK_num);
for i=1:TASK_num
    nb=sum(adj(:,i)==1);
    if nb>0
        R(:,i)=-adj(:,i)/nb;
        R(i,i)=1;
    end
end

L=R*R';
E=eye(TASK_num);

% initialize parameters for training data for each task(size= d*TASK_num)
W=normrnd(0,0.1,d,TASK_num);
U=W;
Y1=zeros(size(W));
% initialize threshold size : (k-1)*TASK_num
pi=ones(k-1,1)*1/k;

% assign each class with same probability
cum_pi=cumsum(pi);
T=log(cum_pi ./ (1-cum_pi));
T= repmat(T, 1, TASK_num);
Y2=zeros(size(T));
V=T;
Y3=zeros(k-2,size(T,2));
% initialize learning rate lambda
lambda_w=lr;
lambda_t=lr;
% admm param
rho=1;

% set max number of iterations for GD on W,T  update (can be shorter if meet stop critira)
Max_iteration=1000;

f1inv=inv(beta*L+2*rho*E);
f2inv=inv(beta*L+rho*E);


for adi= 1: 500

L_old=Inf;
I=zeros(1,Max_iteration);
L_train=zeros(1,Max_iteration);

%% Update W,T via gradient decent

for t = 1 : Max_iteration
    % cross tasks overall loss
    loss=0;

    GW=zeros(d,TASK_num);
    GT=zeros(k-1,TASK_num);

    for i = 1 : TASK_num
        for j = 1 : TRAIN_size
            % calculate what is the model predict is
            if Y_train(i,j)==k
                sig1=1;
                sig2=logsig(T(Y_train(i,j)-1,i)-squeeze(X_train(i,j,:))'*W(:,i));
                dif=SMALL;
            elseif Y_train(i,j)==1
                sig1=logsig(T(Y_train(i,j),i)-squeeze(X_train(i,j,:))'*W(:,i));
                sig2=0;
                dif=SMALL;
            else
                sig1=logsig(T(Y_train(i,j),i)-squeeze(X_train(i,j,:))'*W(:,i));
                sig2=logsig(T(Y_train(i,j)-1,i)-squeeze(X_train(i,j,:))'*W(:,i));
                dif=T(Y_train(i,j)-1,i)-T(Y_train(i,j),i);
            end

            l_cum=-log(sig1-sig2);
            
            if(l_cum==Inf)
                disp('error: inf loss, please decrease learning rate!');
            end
            
            loss=loss+l_cum;
            
            % we want to minimize p_cum w.r.t W and T
            % Gradient Decent

            gradient_w=squeeze(X_train(i,j,:))'*(1-sig1-sig2);

            GW(:,i)=GW(:,i)+gradient_w';

            gradient_t=zeros(k-1,1);
%%
            % Form canonical vector e_yi and e_yi-1
            if Y_train(i,j)==k
                gradient_t(Y_train(i,j)-1)=sig2;
            elseif Y_train(i,j)==1
                gradient_t(Y_train(i,j))=sig1-1;
            else
                gradient_t(Y_train(i,j))=sig1-1+1/(1-exp(-dif));
                gradient_t(Y_train(i,j)-1)=sig2-1+1/(1-exp(dif));
            end
            
            GT(:,i)=GT(:,i)+gradient_t;

        end
    end

    
    GW=1/TRAIN_size*1/TASK_num*GW+Y1+rho*(W-U);

	GT=1/TRAIN_size*1/TASK_num*GT+Y2+rho*(T-V);

    % update parameters
    W=W-lambda_w*GW;
    T=T-lambda_t*GT;
   
    I(t)=t;
    L_train(t)=1/TRAIN_size*1/TASK_num*loss+sum(sum(Y1.*(W-U)))+(rho / 2.) * sum(sum((W-U).^2))+sum(sum(Y2.*(T-V)))+(rho / 2.) * sum(sum((T-V).^2));

    if L_old-L_train(t)<1e-2
        break;
    end
    L_old=L_train(t);

end

%% Update U
    U_old=U;
    U=prox_l21(W+1/rho*Y1, alpha/rho);

%% Update V
    V_old=V;
    V(1,:)=1/rho*Y2(1,:)+T(1,:);
    
    for i=2:k-1
        f1=(beta*V(i-1,:)*L+Y2(i,:)+rho*(T(i,:)+V(i-1,:))+Y3(i-1,:))*f1inv;
        f2=(beta*V(i-1,:)*L+Y2(i,:)+rho*T(i,:)+Y3(i-1,:))*f2inv;
        V(i,f1<=V(i-1,:))=f1(f1<=V(i-1,:));
        V(i,f2>V(i-1,:))=f2(f2>V(i-1,:));
    end
    
%% Update Y
    Y1=Y1+rho*(W-U);
    Y2=Y2+rho*(T-V);
    for i=2:k-1
    Y3(i-1,:)=max(Y3(i-1,:)+rho*(V(i-1,:)-V(i,:)),0);
    end
    
%% Compute ADMM residuals and update rho correspondingly
    s=rho*sqrt(sum(sum((U_old-U).^2)))+rho*sqrt(sum(sum((V_old-V).^2)));
    r=sqrt(sum(sum((U-W).^2)))+sqrt(sum(sum((V-T).^2)));
    
    if(r>10*s)
        rho = 2*rho;
        %update inverse matrix for computation of V-update when rho changes
        f1inv=inv(beta*L+2*rho*E);
        f2inv=inv(beta*L+rho*E);
    else
        if(10*r<s)
            rho = rho/2;
            %update inverse matrix for computation of V-update when rho
            %changes
            f1inv=inv(beta*L+2*rho*E);
            f2inv=inv(beta*L+rho*E);
        end
    end
    
    fprintf('ADMM iteration:%d\tr-residual:%e\ts-residual:%e\trho:%f\n',adi,r,s,rho);
    
    % Check for stop criteria
    if (r<1e-2&&s<1e-2)
        break;
    end
    
end

%% The final learned model parameter set
B=cat(1, T, -W);



end


